	; settings for loader
LOADER_REG_0 .rs 1
LOADER_REG_1 .rs 1
LOADER_REG_2 .rs 1
LOADER_REG_3 .rs 1
LOADER_CHR_START_H .rs 1
LOADER_CHR_START_L .rs 1
LOADER_CHR_START_S .rs 1
LOADER_CHR_SIZE_SOURCE .rs 1
LOADER_CHR_SIZE_TARGET .rs 1
LOADER_CHR_COUNT .rs 1
LOADER_MIRRORING .rs 1

CHR_BANK8 .rs 1 ; to select CHR bank
NROM_BANK_H .rs 1 ; to select 16K bank
NROM_BANK_L .rs 1 ; to select 16K bank
NROM_BANK_S .rs 1 ; for loader

loader:
	; sets COOLBOY registers
	lda <LOADER_MIRRORING
	sta $A000
	; set MMC3 registers for NROM mode
	ldx #$00
	ldy #$00
	sty $A001 ; disable PRG RAM
	stx $8000
	sty $8001
	inx
	iny
	iny
	stx $8000
	sty $8001
	inx
	iny
	iny
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001
	inx
	ldy #$00
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001

	lda <LOADER_REG_0
	sta COOLBOY_REG_0
	lda <LOADER_REG_1
	sta COOLBOY_REG_1
	lda <LOADER_REG_2
	sta COOLBOY_REG_2
	lda <LOADER_REG_3
	sta COOLBOY_REG_3

	jmp loader_clean_and_start

loader_end:
	.org $07E0
loader_clean_and_start:
	; clean memory
	lda #$00
	sta <COPY_SOURCE_ADDR
	sta <COPY_SOURCE_ADDR+1
	ldy #$02
	ldx #$07
.loop:
	sta [COPY_SOURCE_ADDR], y
	iny
	bne .loop
	inc <COPY_SOURCE_ADDR+1
	dex
	bne .loop
.loop2:
	sta [COPY_SOURCE_ADDR], y
	iny
	cpy #LOW(.loop2) ; to the very end
	bne .loop2	
	; Start game!
	jmp [$FFFC]
	
	.org loader_end

	; CHR data loader, only one 8KB bank
load_chr:
	lda #$00
	sta $2006
	sta $2006
	ldy #$00
	ldx #$20
load_chr_loop:
	lda [COPY_SOURCE_ADDR], y
	sta $2007
	iny
	bne load_chr_loop
	inc COPY_SOURCE_ADDR+1
	dex
	bne load_chr_loop
	rts

load_all_chr_banks:
	; Maybe CHR is empty?
	lda <LOADER_CHR_SIZE_TARGET
	beq .end
	; Always starting with zero target CHR bank
	lda #0
	sta <CHR_BANK8
	; Source address (lower byte)
	sta <COPY_SOURCE_ADDR
.reinit:
	; Bank counter
	lda #0
	sta <LOADER_CHR_COUNT
	; Source bank - hi
	ldx <LOADER_CHR_START_H
	stx <NROM_BANK_H
	; Source bank - low
	lda <LOADER_CHR_START_L
	sta <NROM_BANK_L
	; Source address - low - 0x80 or 0xC0
	lda <LOADER_CHR_START_S
	sta <COPY_SOURCE_ADDR+1
.loop:
	jsr select_16k_bank ; selecting source and target banks
	jsr load_chr ; loading CHR data from COPY_SOURCE_ADDR to CHR RAM
	lda <COPY_SOURCE_ADDR+1 ; increamenting source address
	and #$A0 ; or from $A000 to $8000
	sta <COPY_SOURCE_ADDR+1
	cmp #$80
	bne .chr_s_not_inc ; if $8000 need to increament source bank
	lda <NROM_BANK_L	
	clc
	adc #1
	sta <NROM_BANK_L
	lda <NROM_BANK_H
	adc #0
	sta <NROM_BANK_H	
.chr_s_not_inc:
	inc <CHR_BANK8 ; also need to increament target CHR bank
	lda <CHR_BANK8
	cmp <LOADER_CHR_SIZE_TARGET ; was it last TARGET bank?
	beq .end ; ...then end
	inc <LOADER_CHR_COUNT ; increamenting counter
	lda <LOADER_CHR_COUNT
	cmp <LOADER_CHR_SIZE_SOURCE ; was it last SOURCE bank?
	beq .reinit ; need to repeat all data then
	jmp .loop	
.end:
	lda #$A5
	sta COOLBOY_REG_0
	lda #$00
	sta COOLBOY_REG_1
	sta COOLBOY_REG_2
	sta COOLBOY_REG_3
	rts

	; select 16KB NROM bank from the whole flash memory
	; $C000-$FFFF is a mirror for $8000-$BFFF 
select_16k_bank:
	;<NROM_BANK_L (1-8)	<NROM_BANK_H (9-11)  <CHR_BANK8 (7-3)

	; set MMC3 registers for NROM mode
	lda <CHR_BANK8
	asl A
	asl A
	asl A
	tay
	ldx #$00
	stx $8000
	sty $8001
	inx
	iny
	iny
	stx $8000
	sty $8001
	inx
	iny
	iny
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001
	inx
	ldy #$00
	stx $8000
	sty $8001
	inx
	iny
	stx $8000
	sty $8001
	
	lda #$00
	sta $A001 ; PRG-RAM protect
	
	sta COOLBOY_REG_3
	lda #$A0
	sta COOLBOY_REG_0
	lda <NROM_BANK_L
	asl A
	sta COOLBOY_REG_1
	lda <NROM_BANK_L
	rol A
	lda <NROM_BANK_H
	rol A
	sta COOLBOY_REG_2

	rts	
